public class ArrList {
    String[] theArray;  // the underlying array that stores the elements
    int eltcount;       // how many elements are in the array
    int end;            // the last USED slot in the array
    int start;          // the first USED slot in the array


    public ArrList(int initialSlots) {
        this.theArray = new String[initialSlots];
        this.eltcount = 0;
        this.end = 0;
        this.start = 0;
    }

    // get an element by its position in the list
    public String get(int index) {

        if ((index >= 0) && (index < this.eltcount)) {
            // We account for start not being at index 0 of theArray (and for elements wrapping around)
            return theArray[(index + this.start) % this.theArray.length];
        }
        throw new IllegalArgumentException("array index " + index + " out of bounds");

    }

    private void resize(int newSize) {
        // make the new array
        String[] newArray = new String[newSize];
        // copy items from the current theArray to newArray
        for (int index = 0; index < this.eltcount; index++) {
            // Note that in the new array, we copy elements to indices 0-this.eltcount - 1,
            // so we use this.get(index) to get the correct element
            newArray[index] = this.get(index);
        }
        this.start = 0; // the elements in the array we copied TO start with 0 (see above)
        if (this.isEmpty()) { // deal with the edge case that the array might be empty when setting this.end
            this.end = 0;
        } else {
            this.end = this.eltcount - 1;
        }
        // change this.theArray to refer to the new, larger array
        this.theArray = newArray;
    }

    private boolean isFull() {
        return this.eltcount == this.theArray.length;
    }

    public boolean isEmpty() {
        return this.eltcount == 0;
    }

    public int size() {
        return this.eltcount;
    }

    public void addLast(String newItem) {
        if (this.isFull()) {
            // add capacity to the array
            this.resize(this.theArray.length * 2);
            // now that the array has room, add the item
            this.addLast(newItem);
        } else {
            if (!(this.isEmpty())) {
                // wraps around to 0 if we've gone beyond the last index of the array
                this.end = (this.end + 1) % this.theArray.length;
            }
            this.eltcount = this.eltcount + 1;
            this.theArray[this.end] = newItem;
        }
    }

    public void addFirst(String newItem) {
        if (this.isFull()) {
            this.resize(this.theArray.length * 2);
            this.addFirst(newItem);
        } else {
            if (!this.isEmpty()) {
                // wraps around to the last index of the array if we've gone beyond 0
                this.start = (this.start - 1 + this.theArray.length) % this.theArray.length;
            }
            this.eltcount = this.eltcount + 1;
            this.theArray[this.start] = newItem;
        }
    }
}
